////////////////////////////////////////////////////////////////////////
// Class:       MonitorTDCs
// Module Type: analyzer
// File:        MonitorTDCs_module.cc
//
// Generated at Wed Oct 30 07:53:11 2013 by Alessandro Razeto using artmod
// from art v1_03_08.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "artdaq-core/Data/Fragments.hh"
#include "darksidecore/Data/V1190Fragment.hh"

#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TH1I.h"
#include "TH1F.h"

#include <numeric>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>

namespace ds50 {
  class MonitorTDCs : public art::EDAnalyzer {
    public:
      explicit MonitorTDCs(fhicl::ParameterSet const & p);
      virtual ~MonitorTDCs();

      void analyze(art::Event const & e) override;
      void beginRun(art::Run const &) override;

    private:
      std::unique_ptr<TCanvas> canvas_;
      std::unique_ptr<TH1I> channel_occupancy_[2], event_size_[2];

      art::RunNumber_t current_run_;
  };
}

ds50::MonitorTDCs::MonitorTDCs(fhicl::ParameterSet const &ps) :
  EDAnalyzer(ps),current_run_(0) { }

ds50::MonitorTDCs::~MonitorTDCs() { }

void ds50::MonitorTDCs::analyze(art::Event const & e) {
  art::Handle<artdaq::Fragments> v1190s;
  e.getByLabel ("daq", "V1190", v1190s);
  if (!v1190s.isValid () || v1190s->size() != 2) return;

  std::vector<int> ids;
  for (size_t i = 0; i < v1190s->size(); ++i) ids.push_back (V1190Fragment((*v1190s)[i]).dataBegin ()->geo ());
  std::sort (ids.begin (), ids.end ());

  for (size_t i = 0; i < v1190s->size(); ++i) {
    const auto& frag((*v1190s)[i]);

    V1190Fragment b(frag);
    int idx = (b.dataBegin ()->geo () == ids[0]) ? 0 : 1;

    int size = 0;
    for (const V1190Fragment::Word *w = b.dataBegin (); w != b.dataEnd (); ++w) {
      if (w->type () == V1190Fragment::Word::measurement_w && !w->trailing_edge ()) {
	channel_occupancy_[idx]->Fill (w->channel ());
	size ++;
      }
    }
    event_size_[idx]->Fill (size);
  } 

  if (!(int(event_size_[0]->GetEntries ()) % 50)) {
    for (int i = 1; i <= 4; i++) canvas_->cd (i)->Modified (); 
    canvas_->Update (); 
  }
}


void ds50::MonitorTDCs::beginRun(art::Run const &e) {
  if (e.run () == current_run_) return;
  current_run_ = e.run ();

  canvas_ = std::unique_ptr<TCanvas>(new TCanvas("TDC status"));
  ((TRootCanvas*)canvas_->GetCanvasImp ())->DontCallClose ();
  canvas_ -> Divide(1,4); 

  canvas_ -> cd(1); 
  event_size_[0] = std::unique_ptr<TH1I>(new TH1I("veto_tdc1_size", "Event size for VETO TDC 1", 200, 0, 200));
  event_size_[0]->SetBit (TH1::kCanRebin);
  event_size_[0]->Draw ();

  canvas_ -> cd(2);
  event_size_[1] = std::unique_ptr<TH1I>(new TH1I("veto_tdc2_size", "Event size for VETO TDC 2", 200, 0, 200));
  event_size_[1]->SetBit (TH1::kCanRebin);
  event_size_[1]->Draw ();

  canvas_ -> cd(3);  
  channel_occupancy_[0] = std::unique_ptr<TH1I>(new TH1I("veto_tdc1_occupancy", "Channel Occupancy Veto TDC 1", 128, -0.5, 127.5));
  channel_occupancy_[0]->Draw ();

  canvas_ -> cd(4);  
  channel_occupancy_[1] = std::unique_ptr<TH1I>(new TH1I("veto_tdc2_occupancy", "Channel Occupancy Veto TDC 2", 128, -0.5, 127.5));
  channel_occupancy_[1]->Draw ();

  canvas_->Update();
}

DEFINE_ART_MODULE(ds50::MonitorTDCs)
