////////////////////////////////////////////////////////////////////////
// Class:       MonitorTrigger
// Module Type: analyzer
// File:        MonitorTrigger_module.cc
//
// Generated at Tue Apr  9 14:28:00 2013 by Nicola Rossi using artmod
// from art v1_03_08.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "artdaq-core/Data/Fragments.hh"
#include "darksidecore/Data/V1495Fragment.hh"

#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TH1I.h"
#include "TH1F.h"

#include <numeric>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>

namespace ds50 {
  class MonitorTrigger : public art::EDAnalyzer {
    public:
      explicit MonitorTrigger(fhicl::ParameterSet const & p);
      virtual ~MonitorTrigger();

      void analyze(art::Event const & e) override;
      void beginRun(art::Run const &) override;

    private:
      std::unique_ptr<TCanvas> canvas_;
      std::map<V1495Fragment::trigger_types, std::unique_ptr<TH1F>>  hists_;
      std::unique_ptr<TH1I> hist_ch_;

      time_t start_time_;
      std::map<V1495Fragment::trigger_types, int> current_counts_;

      art::RunNumber_t current_run_;
  };
}

ds50::MonitorTrigger::MonitorTrigger(fhicl::ParameterSet const &p):
  art::EDAnalyzer(p), start_time_(0), current_run_(0) { }

ds50::MonitorTrigger::~MonitorTrigger() { }

void ds50::MonitorTrigger::analyze(art::Event const & e) {
  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid ()) return;

  if(v1495->size() != 1) {
    mf::LogError ("MonitorTrigger") << v1495->size() << " fragment for V1495";
    return; 
  }

  if ((*v1495)[0].hasMetadata()) {
    V1495Fragment::metadata const* md = (*v1495)[0].metadata<V1495Fragment::metadata>();
    time_t trigger_time =  md -> trigger_time.tv_sec;
    if (!start_time_) { 
      start_time_ = trigger_time;
      return;
    }

    V1495Fragment b((*v1495)[0]);
    V1495Fragment::trigger_types tt = b.trigger_type ();
    if (!hists_[tt]) return;

    double dt = difftime(trigger_time, start_time_);
    if (dt >= hists_[tt]->GetBinLowEdge(hists_[tt]->GetNbinsX ())) {
      canvas_ -> cd(1); // -> SetLogy ();
      bool first = true;
      for (auto &x: hists_) {
	std::unique_ptr<TH1F> y = std::move (x.second);
	x.second = std::unique_ptr<TH1F>(new TH1F("TriggerRate","Trigger Rate [Hz] ",y->GetNbinsX () * 2, 0, y->GetBinLowEdge(y->GetNbinsX () + 1) * 2));
	for (int i = 0; i < y->GetNbinsX (); i++) x.second->Fill (y->GetBinCenter (i + 1), y->GetBinContent (i + 1));
	x.second->SetLineColor (y->GetLineColor ());
	x.second->Draw(first ? "" : "SAME");
	first = false;
      }
    }
    hists_[tt]->Fill (difftime(trigger_time, start_time_), 1. / hists_[tt]->GetBinWidth (1));

    if (tt == V1495Fragment::tpc || tt == V1495Fragment::tpc_high || tt == V1495Fragment::tpc_low) 
      for (int ch = 0; ch < 40; ch++) if(b.channel_in_trigger(ch)) hist_ch_ -> Fill(ch);

    if (!((*v1495)[0].sequenceID () % 50)) {
      double max = 0;
      for (auto &x : hists_) {
        if (x.second.get() != 0) max = std::max<double> (max, x.second->GetBinContent (x.second->GetMaximumBin ()));
      }
      for (auto &x : hists_) {
        if (x.second.get() != 0) x.second->SetAxisRange (0, max * 1.2, "Y");
      }
      canvas_->cd (1)-> Modified();
      canvas_->cd (2)-> Modified();
      canvas_->Update();  
    }
  } 
}


void ds50::MonitorTrigger::beginRun(art::Run const &e) {
  if (e.run () == current_run_) return;
  current_run_ = e.run ();

  canvas_ = 0;
  hist_ch_ = 0;
  hists_.clear ();

  canvas_ = std::unique_ptr<TCanvas>(new TCanvas("TriggerRate"));
  ((TRootCanvas*)canvas_->GetCanvasImp ())->DontCallClose ();
  canvas_ -> Divide(1,2); 

  canvas_ -> cd(1); // -> SetLogy ();
  V1495Fragment::trigger_types tts[] = { V1495Fragment::tpc, V1495Fragment::tpc_high, V1495Fragment::tpc_low, V1495Fragment::pulser, V1495Fragment::random, V1495Fragment::laser, V1495Fragment::external };
  int color = 1;
  for (auto tt : tts) {
    hists_[tt] = std::unique_ptr<TH1F>(new TH1F("TriggerRate","Trigger Rate [Hz] ",20,0,600));
    hists_[tt] -> SetLineColor ((tt == V1495Fragment::tpc_low) ? 1 : color);
    hists_[tt] -> Draw((tt == V1495Fragment::tpc) ? "" : "SAME");
    hists_[tt] -> SetBit(TH1::kCanRebin);
    color++;
  }

  canvas_ -> cd(2);  
  hist_ch_ = std::unique_ptr<TH1I>(new TH1I("Channels","Channel Occupancy in Trigger",40,-0.5,39.5));    
  hist_ch_ -> SetNdivisions(39);
  hist_ch_ -> SetStats(0);         
  hist_ch_ -> Draw();

  canvas_->Update();

  start_time_ = 0;
}

DEFINE_ART_MODULE(ds50::MonitorTrigger)
