#ifndef OnlineEvent_hh
#define OnlineEvent_hh

#include <vector>

namespace ds50 {
  namespace Online {
    struct RunInfo {
      int run_number, event_number;
      int run_time_s, live_time_s, n_events;
    };

    struct Waveforms {
      std::vector<float> wf[40];
      std::vector<float> times_ns;
      std::vector<float> sum;
      float sample_ns;
      int pre_samples, post_samples;
    };

    struct Peak {
      int time_ns;
      float height;      
    };

    struct Cluster {	
      int start_time_ns, stop_time_ns, cluster_rep; 
      float mean_time_ns, rms_time_ns;
      float charge, abs_charge;
      enum psd_types { f80 = 0, f85, f90, f95, f100, f105, f110, f115, f120 };
      float psd[f120 + 1];
      std::vector<Peak> peaks;    
      struct PMT {
	float charge, abs_charge;
	float psd[f120 + 1];
      } pmts[40];

    };

    struct Clusters {
      double gps_time;
      std::vector<Cluster> clusters;
    };
   
    struct Position {
      float total_charge;
      float normalized_charge[40];
      float x_col, y_col;
      float x_ls, y_ls, lkl_ls;
      float z;
    };
      
    struct Positions {
      std::vector<Position> positions;  
    }; 

    struct TDCCluster {	
      int start_time_ns, stop_time_ns;
      float mean_time_ns;
      short n_hits, n_long_hits;
      float long_hits_avg;
      unsigned char multiplicity[8];
    };

    struct TDCClusters {
      std::vector<TDCCluster> ls_clusters;
      std::vector<TDCCluster> wc_clusters;
    };
  }
}

#endif
