////////////////////////////////////////////////////////////////////////
// Class:       OnlineRunInfo
// Module Type: producer
// File:        OnlineRunInfo_module.cc
//
// Generated at Mon Aug 19 10:52:46 2013 by Alessandro Razeto & Nicola Rossi using artmod
// from cetpkgsupport v1_02_00.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "darksidecore/ArtServices/DBInterface.hh"

#include "darkmon/ArtModules/OnlineEvent.hh"

namespace ds50 {
  class OnlineRunInfo : public art::EDProducer {
    public:
      explicit OnlineRunInfo(fhicl::ParameterSet const & p);
      virtual ~OnlineRunInfo();

      void produce(art::Event & e) override;
      void beginRun(art::Run &) override;

    private:
      int run_time_s_, live_time_s_, n_events_;
  };
}

ds50::OnlineRunInfo::OnlineRunInfo(fhicl::ParameterSet const &) {
  produces<ds50::Online::RunInfo> ();
}

ds50::OnlineRunInfo::~OnlineRunInfo() {}

void ds50::OnlineRunInfo::produce(art::Event & e) {
  std::unique_ptr<ds50::Online::RunInfo> rinfo(new ds50::Online::RunInfo);
  rinfo->run_time_s = run_time_s_;
  rinfo->live_time_s = live_time_s_;
  rinfo->n_events = n_events_;
  rinfo->event_number = e.event ();
  rinfo->run_number = e.run ();

  e.put(std::move(rinfo));
}

void ds50::OnlineRunInfo::beginRun(art::Run &) {
  art::ServiceHandle<ds50::DBInterface> dbi;
  ds50::db::result r = dbi->current ("shadowside.livetime");
  run_time_s_ = r.get<float> ("runtime_daq", 0);
  live_time_s_ = r.get<float> ("livetime_v1495", 0);
  n_events_ = r.get<int> ("events", 0);
}

DEFINE_ART_MODULE(ds50::OnlineRunInfo)
